/****************************************************************************
**
** Copyright (C) Qxt Foundation. Some rights reserved.
**
** This file is part of the QxtDesigner module of the Qxt library.
**
** This library is free software; you can redistribute it and/or modify it
** under the terms of the Common Public License, version 1.0, as published by
** IBM.
**
** This file is provided "AS IS", without WARRANTIES OR CONDITIONS OF ANY
** KIND, EITHER EXPRESS OR IMPLIED INCLUDING, WITHOUT LIMITATION, ANY
** WARRANTIES OR CONDITIONS OF TITLE, NON-INFRINGEMENT, MERCHANTABILITY OR
** FITNESS FOR A PARTICULAR PURPOSE.
**
** You should have received a copy of the CPL along with this file.
** See the LICENSE file and the cpl1.0.txt file included with the source
** distribution for more information. If you did not receive a copy of the
** license, contact the Qxt Foundation.
**
** <http://libqxt.org>  <foundation@libqxt.org>
**
****************************************************************************/

//
// Generated on Tue Jun 3 23:21:31 2008
//  for Qt 4.4.0
//

#if 0
static const char* language_strings = {
  QT_TRANSLATE_NOOP("QLocale", "C"),
  QT_TRANSLATE_NOOP("QLocale", "Abkhazian"),
  QT_TRANSLATE_NOOP("QLocale", "Afan"),
  QT_TRANSLATE_NOOP("QLocale", "Afar"),
  QT_TRANSLATE_NOOP("QLocale", "Afrikaans"),
  QT_TRANSLATE_NOOP("QLocale", "Albanian"),
  QT_TRANSLATE_NOOP("QLocale", "Amharic"),
  QT_TRANSLATE_NOOP("QLocale", "Arabic"),
  QT_TRANSLATE_NOOP("QLocale", "Armenian"),
  QT_TRANSLATE_NOOP("QLocale", "Assamese"),
  QT_TRANSLATE_NOOP("QLocale", "Aymara"),
  QT_TRANSLATE_NOOP("QLocale", "Azerbaijani"),
  QT_TRANSLATE_NOOP("QLocale", "Bashkir"),
  QT_TRANSLATE_NOOP("QLocale", "Basque"),
  QT_TRANSLATE_NOOP("QLocale", "Bengali"),
  QT_TRANSLATE_NOOP("QLocale", "Bhutani"),
  QT_TRANSLATE_NOOP("QLocale", "Bihari"),
  QT_TRANSLATE_NOOP("QLocale", "Bislama"),
  QT_TRANSLATE_NOOP("QLocale", "Breton"),
  QT_TRANSLATE_NOOP("QLocale", "Bulgarian"),
  QT_TRANSLATE_NOOP("QLocale", "Burmese"),
  QT_TRANSLATE_NOOP("QLocale", "Byelorussian"),
  QT_TRANSLATE_NOOP("QLocale", "Cambodian"),
  QT_TRANSLATE_NOOP("QLocale", "Catalan"),
  QT_TRANSLATE_NOOP("QLocale", "Chinese"),
  QT_TRANSLATE_NOOP("QLocale", "Corsican"),
  QT_TRANSLATE_NOOP("QLocale", "Croatian"),
  QT_TRANSLATE_NOOP("QLocale", "Czech"),
  QT_TRANSLATE_NOOP("QLocale", "Danish"),
  QT_TRANSLATE_NOOP("QLocale", "Dutch"),
  QT_TRANSLATE_NOOP("QLocale", "English"),
  QT_TRANSLATE_NOOP("QLocale", "Esperanto"),
  QT_TRANSLATE_NOOP("QLocale", "Estonian"),
  QT_TRANSLATE_NOOP("QLocale", "Faroese"),
  QT_TRANSLATE_NOOP("QLocale", "FijiLanguage"),
  QT_TRANSLATE_NOOP("QLocale", "Finnish"),
  QT_TRANSLATE_NOOP("QLocale", "French"),
  QT_TRANSLATE_NOOP("QLocale", "Frisian"),
  QT_TRANSLATE_NOOP("QLocale", "Gaelic"),
  QT_TRANSLATE_NOOP("QLocale", "Galician"),
  QT_TRANSLATE_NOOP("QLocale", "Georgian"),
  QT_TRANSLATE_NOOP("QLocale", "German"),
  QT_TRANSLATE_NOOP("QLocale", "Greek"),
  QT_TRANSLATE_NOOP("QLocale", "Greenlandic"),
  QT_TRANSLATE_NOOP("QLocale", "Guarani"),
  QT_TRANSLATE_NOOP("QLocale", "Gujarati"),
  QT_TRANSLATE_NOOP("QLocale", "Hausa"),
  QT_TRANSLATE_NOOP("QLocale", "Hebrew"),
  QT_TRANSLATE_NOOP("QLocale", "Hindi"),
  QT_TRANSLATE_NOOP("QLocale", "Hungarian"),
  QT_TRANSLATE_NOOP("QLocale", "Icelandic"),
  QT_TRANSLATE_NOOP("QLocale", "Indonesian"),
  QT_TRANSLATE_NOOP("QLocale", "Interlingua"),
  QT_TRANSLATE_NOOP("QLocale", "Interlingue"),
  QT_TRANSLATE_NOOP("QLocale", "Inuktitut"),
  QT_TRANSLATE_NOOP("QLocale", "Inupiak"),
  QT_TRANSLATE_NOOP("QLocale", "Irish"),
  QT_TRANSLATE_NOOP("QLocale", "Italian"),
  QT_TRANSLATE_NOOP("QLocale", "Japanese"),
  QT_TRANSLATE_NOOP("QLocale", "Javanese"),
  QT_TRANSLATE_NOOP("QLocale", "Kannada"),
  QT_TRANSLATE_NOOP("QLocale", "Kashmiri"),
  QT_TRANSLATE_NOOP("QLocale", "Kazakh"),
  QT_TRANSLATE_NOOP("QLocale", "Kinyarwanda"),
  QT_TRANSLATE_NOOP("QLocale", "Kirghiz"),
  QT_TRANSLATE_NOOP("QLocale", "Korean"),
  QT_TRANSLATE_NOOP("QLocale", "Kurdish"),
  QT_TRANSLATE_NOOP("QLocale", "Kurundi"),
  QT_TRANSLATE_NOOP("QLocale", "Laothian"),
  QT_TRANSLATE_NOOP("QLocale", "Latin"),
  QT_TRANSLATE_NOOP("QLocale", "Latvian"),
  QT_TRANSLATE_NOOP("QLocale", "Lingala"),
  QT_TRANSLATE_NOOP("QLocale", "Lithuanian"),
  QT_TRANSLATE_NOOP("QLocale", "Macedonian"),
  QT_TRANSLATE_NOOP("QLocale", "Malagasy"),
  QT_TRANSLATE_NOOP("QLocale", "Malay"),
  QT_TRANSLATE_NOOP("QLocale", "Malayalam"),
  QT_TRANSLATE_NOOP("QLocale", "Maltese"),
  QT_TRANSLATE_NOOP("QLocale", "Maori"),
  QT_TRANSLATE_NOOP("QLocale", "Marathi"),
  QT_TRANSLATE_NOOP("QLocale", "Moldavian"),
  QT_TRANSLATE_NOOP("QLocale", "Mongolian"),
  QT_TRANSLATE_NOOP("QLocale", "NauruLanguage"),
  QT_TRANSLATE_NOOP("QLocale", "Nepali"),
  QT_TRANSLATE_NOOP("QLocale", "Norwegian"),
  QT_TRANSLATE_NOOP("QLocale", "NorwegianBokmal"),
  QT_TRANSLATE_NOOP("QLocale", "Occitan"),
  QT_TRANSLATE_NOOP("QLocale", "Oriya"),
  QT_TRANSLATE_NOOP("QLocale", "Pashto"),
  QT_TRANSLATE_NOOP("QLocale", "Persian"),
  QT_TRANSLATE_NOOP("QLocale", "Polish"),
  QT_TRANSLATE_NOOP("QLocale", "Portuguese"),
  QT_TRANSLATE_NOOP("QLocale", "Punjabi"),
  QT_TRANSLATE_NOOP("QLocale", "Quechua"),
  QT_TRANSLATE_NOOP("QLocale", "RhaetoRomance"),
  QT_TRANSLATE_NOOP("QLocale", "Romanian"),
  QT_TRANSLATE_NOOP("QLocale", "Russian"),
  QT_TRANSLATE_NOOP("QLocale", "Samoan"),
  QT_TRANSLATE_NOOP("QLocale", "Sangho"),
  QT_TRANSLATE_NOOP("QLocale", "Sanskrit"),
  QT_TRANSLATE_NOOP("QLocale", "Serbian"),
  QT_TRANSLATE_NOOP("QLocale", "SerboCroatian"),
  QT_TRANSLATE_NOOP("QLocale", "Sesotho"),
  QT_TRANSLATE_NOOP("QLocale", "Setswana"),
  QT_TRANSLATE_NOOP("QLocale", "Shona"),
  QT_TRANSLATE_NOOP("QLocale", "Sindhi"),
  QT_TRANSLATE_NOOP("QLocale", "Singhalese"),
  QT_TRANSLATE_NOOP("QLocale", "Siswati"),
  QT_TRANSLATE_NOOP("QLocale", "Slovak"),
  QT_TRANSLATE_NOOP("QLocale", "Slovenian"),
  QT_TRANSLATE_NOOP("QLocale", "Somali"),
  QT_TRANSLATE_NOOP("QLocale", "Spanish"),
  QT_TRANSLATE_NOOP("QLocale", "Sundanese"),
  QT_TRANSLATE_NOOP("QLocale", "Swahili"),
  QT_TRANSLATE_NOOP("QLocale", "Swedish"),
  QT_TRANSLATE_NOOP("QLocale", "Tagalog"),
  QT_TRANSLATE_NOOP("QLocale", "Tajik"),
  QT_TRANSLATE_NOOP("QLocale", "Tamil"),
  QT_TRANSLATE_NOOP("QLocale", "Tatar"),
  QT_TRANSLATE_NOOP("QLocale", "Telugu"),
  QT_TRANSLATE_NOOP("QLocale", "Thai"),
  QT_TRANSLATE_NOOP("QLocale", "Tibetan"),
  QT_TRANSLATE_NOOP("QLocale", "Tigrinya"),
  QT_TRANSLATE_NOOP("QLocale", "TongaLanguage"),
  QT_TRANSLATE_NOOP("QLocale", "Tsonga"),
  QT_TRANSLATE_NOOP("QLocale", "Turkish"),
  QT_TRANSLATE_NOOP("QLocale", "Turkmen"),
  QT_TRANSLATE_NOOP("QLocale", "Twi"),
  QT_TRANSLATE_NOOP("QLocale", "Uigur"),
  QT_TRANSLATE_NOOP("QLocale", "Ukrainian"),
  QT_TRANSLATE_NOOP("QLocale", "Urdu"),
  QT_TRANSLATE_NOOP("QLocale", "Uzbek"),
  QT_TRANSLATE_NOOP("QLocale", "Vietnamese"),
  QT_TRANSLATE_NOOP("QLocale", "Volapuk"),
  QT_TRANSLATE_NOOP("QLocale", "Welsh"),
  QT_TRANSLATE_NOOP("QLocale", "Wolof"),
  QT_TRANSLATE_NOOP("QLocale", "Xhosa"),
  QT_TRANSLATE_NOOP("QLocale", "Yiddish"),
  QT_TRANSLATE_NOOP("QLocale", "Yoruba"),
  QT_TRANSLATE_NOOP("QLocale", "Zhuang"),
  QT_TRANSLATE_NOOP("QLocale", "Zulu"),
  QT_TRANSLATE_NOOP("QLocale", "NorwegianNynorsk"),
  QT_TRANSLATE_NOOP("QLocale", "Nynorsk"),
  QT_TRANSLATE_NOOP("QLocale", "Bosnian"),
  QT_TRANSLATE_NOOP("QLocale", "Divehi"),
  QT_TRANSLATE_NOOP("QLocale", "Manx"),
  QT_TRANSLATE_NOOP("QLocale", "Cornish"),
  QT_TRANSLATE_NOOP("QLocale", "Akan"),
  QT_TRANSLATE_NOOP("QLocale", "Konkani"),
  QT_TRANSLATE_NOOP("QLocale", "Ga"),
  QT_TRANSLATE_NOOP("QLocale", "Igbo"),
  QT_TRANSLATE_NOOP("QLocale", "Kamba"),
  QT_TRANSLATE_NOOP("QLocale", "Syriac"),
  QT_TRANSLATE_NOOP("QLocale", "Blin"),
  QT_TRANSLATE_NOOP("QLocale", "Geez"),
  QT_TRANSLATE_NOOP("QLocale", "Koro"),
  QT_TRANSLATE_NOOP("QLocale", "Sidamo"),
  QT_TRANSLATE_NOOP("QLocale", "Atsam"),
  QT_TRANSLATE_NOOP("QLocale", "Tigre"),
  QT_TRANSLATE_NOOP("QLocale", "Jju"),
  QT_TRANSLATE_NOOP("QLocale", "Friulian"),
  QT_TRANSLATE_NOOP("QLocale", "Venda"),
  QT_TRANSLATE_NOOP("QLocale", "Ewe"),
  QT_TRANSLATE_NOOP("QLocale", "Walamo"),
  QT_TRANSLATE_NOOP("QLocale", "Hawaiian"),
  QT_TRANSLATE_NOOP("QLocale", "Tyap"),
};
static const char* countries_strings = {
  QT_TRANSLATE_NOOP("QLocale", "AnyCountry"),
  QT_TRANSLATE_NOOP("QLocale", "Afghanistan"),
  QT_TRANSLATE_NOOP("QLocale", "Albania"),
  QT_TRANSLATE_NOOP("QLocale", "Algeria"),
  QT_TRANSLATE_NOOP("QLocale", "AmericanSamoa"),
  QT_TRANSLATE_NOOP("QLocale", "Andorra"),
  QT_TRANSLATE_NOOP("QLocale", "Angola"),
  QT_TRANSLATE_NOOP("QLocale", "Anguilla"),
  QT_TRANSLATE_NOOP("QLocale", "Antarctica"),
  QT_TRANSLATE_NOOP("QLocale", "AntiguaAndBarbuda"),
  QT_TRANSLATE_NOOP("QLocale", "Argentina"),
  QT_TRANSLATE_NOOP("QLocale", "Armenia"),
  QT_TRANSLATE_NOOP("QLocale", "Aruba"),
  QT_TRANSLATE_NOOP("QLocale", "Australia"),
  QT_TRANSLATE_NOOP("QLocale", "Austria"),
  QT_TRANSLATE_NOOP("QLocale", "Azerbaijan"),
  QT_TRANSLATE_NOOP("QLocale", "Bahamas"),
  QT_TRANSLATE_NOOP("QLocale", "Bahrain"),
  QT_TRANSLATE_NOOP("QLocale", "Bangladesh"),
  QT_TRANSLATE_NOOP("QLocale", "Barbados"),
  QT_TRANSLATE_NOOP("QLocale", "Belarus"),
  QT_TRANSLATE_NOOP("QLocale", "Belgium"),
  QT_TRANSLATE_NOOP("QLocale", "Belize"),
  QT_TRANSLATE_NOOP("QLocale", "Benin"),
  QT_TRANSLATE_NOOP("QLocale", "Bermuda"),
  QT_TRANSLATE_NOOP("QLocale", "Bhutan"),
  QT_TRANSLATE_NOOP("QLocale", "Bolivia"),
  QT_TRANSLATE_NOOP("QLocale", "BosniaAndHerzegowina"),
  QT_TRANSLATE_NOOP("QLocale", "Botswana"),
  QT_TRANSLATE_NOOP("QLocale", "BouvetIsland"),
  QT_TRANSLATE_NOOP("QLocale", "Brazil"),
  QT_TRANSLATE_NOOP("QLocale", "BritishIndianOceanTerritory"),
  QT_TRANSLATE_NOOP("QLocale", "BruneiDarussalam"),
  QT_TRANSLATE_NOOP("QLocale", "Bulgaria"),
  QT_TRANSLATE_NOOP("QLocale", "BurkinaFaso"),
  QT_TRANSLATE_NOOP("QLocale", "Burundi"),
  QT_TRANSLATE_NOOP("QLocale", "Cambodia"),
  QT_TRANSLATE_NOOP("QLocale", "Cameroon"),
  QT_TRANSLATE_NOOP("QLocale", "Canada"),
  QT_TRANSLATE_NOOP("QLocale", "CapeVerde"),
  QT_TRANSLATE_NOOP("QLocale", "CaymanIslands"),
  QT_TRANSLATE_NOOP("QLocale", "CentralAfricanRepublic"),
  QT_TRANSLATE_NOOP("QLocale", "Chad"),
  QT_TRANSLATE_NOOP("QLocale", "Chile"),
  QT_TRANSLATE_NOOP("QLocale", "China"),
  QT_TRANSLATE_NOOP("QLocale", "ChristmasIsland"),
  QT_TRANSLATE_NOOP("QLocale", "CocosIslands"),
  QT_TRANSLATE_NOOP("QLocale", "Colombia"),
  QT_TRANSLATE_NOOP("QLocale", "Comoros"),
  QT_TRANSLATE_NOOP("QLocale", "DemocraticRepublicOfCongo"),
  QT_TRANSLATE_NOOP("QLocale", "PeoplesRepublicOfCongo"),
  QT_TRANSLATE_NOOP("QLocale", "CookIslands"),
  QT_TRANSLATE_NOOP("QLocale", "CostaRica"),
  QT_TRANSLATE_NOOP("QLocale", "IvoryCoast"),
  QT_TRANSLATE_NOOP("QLocale", "Croatia"),
  QT_TRANSLATE_NOOP("QLocale", "Cuba"),
  QT_TRANSLATE_NOOP("QLocale", "Cyprus"),
  QT_TRANSLATE_NOOP("QLocale", "CzechRepublic"),
  QT_TRANSLATE_NOOP("QLocale", "Denmark"),
  QT_TRANSLATE_NOOP("QLocale", "Djibouti"),
  QT_TRANSLATE_NOOP("QLocale", "Dominica"),
  QT_TRANSLATE_NOOP("QLocale", "DominicanRepublic"),
  QT_TRANSLATE_NOOP("QLocale", "EastTimor"),
  QT_TRANSLATE_NOOP("QLocale", "Ecuador"),
  QT_TRANSLATE_NOOP("QLocale", "Egypt"),
  QT_TRANSLATE_NOOP("QLocale", "ElSalvador"),
  QT_TRANSLATE_NOOP("QLocale", "EquatorialGuinea"),
  QT_TRANSLATE_NOOP("QLocale", "Eritrea"),
  QT_TRANSLATE_NOOP("QLocale", "Estonia"),
  QT_TRANSLATE_NOOP("QLocale", "Ethiopia"),
  QT_TRANSLATE_NOOP("QLocale", "FalklandIslands"),
  QT_TRANSLATE_NOOP("QLocale", "FaroeIslands"),
  QT_TRANSLATE_NOOP("QLocale", "FijiCountry"),
  QT_TRANSLATE_NOOP("QLocale", "Finland"),
  QT_TRANSLATE_NOOP("QLocale", "France"),
  QT_TRANSLATE_NOOP("QLocale", "MetropolitanFrance"),
  QT_TRANSLATE_NOOP("QLocale", "FrenchGuiana"),
  QT_TRANSLATE_NOOP("QLocale", "FrenchPolynesia"),
  QT_TRANSLATE_NOOP("QLocale", "FrenchSouthernTerritories"),
  QT_TRANSLATE_NOOP("QLocale", "Gabon"),
  QT_TRANSLATE_NOOP("QLocale", "Gambia"),
  QT_TRANSLATE_NOOP("QLocale", "Georgia"),
  QT_TRANSLATE_NOOP("QLocale", "Germany"),
  QT_TRANSLATE_NOOP("QLocale", "Ghana"),
  QT_TRANSLATE_NOOP("QLocale", "Gibraltar"),
  QT_TRANSLATE_NOOP("QLocale", "Greece"),
  QT_TRANSLATE_NOOP("QLocale", "Greenland"),
  QT_TRANSLATE_NOOP("QLocale", "Grenada"),
  QT_TRANSLATE_NOOP("QLocale", "Guadeloupe"),
  QT_TRANSLATE_NOOP("QLocale", "Guam"),
  QT_TRANSLATE_NOOP("QLocale", "Guatemala"),
  QT_TRANSLATE_NOOP("QLocale", "Guinea"),
  QT_TRANSLATE_NOOP("QLocale", "GuineaBissau"),
  QT_TRANSLATE_NOOP("QLocale", "Guyana"),
  QT_TRANSLATE_NOOP("QLocale", "Haiti"),
  QT_TRANSLATE_NOOP("QLocale", "HeardAndMcDonaldIslands"),
  QT_TRANSLATE_NOOP("QLocale", "Honduras"),
  QT_TRANSLATE_NOOP("QLocale", "HongKong"),
  QT_TRANSLATE_NOOP("QLocale", "Hungary"),
  QT_TRANSLATE_NOOP("QLocale", "Iceland"),
  QT_TRANSLATE_NOOP("QLocale", "India"),
  QT_TRANSLATE_NOOP("QLocale", "Indonesia"),
  QT_TRANSLATE_NOOP("QLocale", "Iran"),
  QT_TRANSLATE_NOOP("QLocale", "Iraq"),
  QT_TRANSLATE_NOOP("QLocale", "Ireland"),
  QT_TRANSLATE_NOOP("QLocale", "Israel"),
  QT_TRANSLATE_NOOP("QLocale", "Italy"),
  QT_TRANSLATE_NOOP("QLocale", "Jamaica"),
  QT_TRANSLATE_NOOP("QLocale", "Japan"),
  QT_TRANSLATE_NOOP("QLocale", "Jordan"),
  QT_TRANSLATE_NOOP("QLocale", "Kazakhstan"),
  QT_TRANSLATE_NOOP("QLocale", "Kenya"),
  QT_TRANSLATE_NOOP("QLocale", "Kiribati"),
  QT_TRANSLATE_NOOP("QLocale", "DemocraticRepublicOfKorea"),
  QT_TRANSLATE_NOOP("QLocale", "RepublicOfKorea"),
  QT_TRANSLATE_NOOP("QLocale", "Kuwait"),
  QT_TRANSLATE_NOOP("QLocale", "Kyrgyzstan"),
  QT_TRANSLATE_NOOP("QLocale", "Lao"),
  QT_TRANSLATE_NOOP("QLocale", "Latvia"),
  QT_TRANSLATE_NOOP("QLocale", "Lebanon"),
  QT_TRANSLATE_NOOP("QLocale", "Lesotho"),
  QT_TRANSLATE_NOOP("QLocale", "Liberia"),
  QT_TRANSLATE_NOOP("QLocale", "LibyanArabJamahiriya"),
  QT_TRANSLATE_NOOP("QLocale", "Liechtenstein"),
  QT_TRANSLATE_NOOP("QLocale", "Lithuania"),
  QT_TRANSLATE_NOOP("QLocale", "Luxembourg"),
  QT_TRANSLATE_NOOP("QLocale", "Macau"),
  QT_TRANSLATE_NOOP("QLocale", "Macedonia"),
  QT_TRANSLATE_NOOP("QLocale", "Madagascar"),
  QT_TRANSLATE_NOOP("QLocale", "Malawi"),
  QT_TRANSLATE_NOOP("QLocale", "Malaysia"),
  QT_TRANSLATE_NOOP("QLocale", "Maldives"),
  QT_TRANSLATE_NOOP("QLocale", "Mali"),
  QT_TRANSLATE_NOOP("QLocale", "Malta"),
  QT_TRANSLATE_NOOP("QLocale", "MarshallIslands"),
  QT_TRANSLATE_NOOP("QLocale", "Martinique"),
  QT_TRANSLATE_NOOP("QLocale", "Mauritania"),
  QT_TRANSLATE_NOOP("QLocale", "Mauritius"),
  QT_TRANSLATE_NOOP("QLocale", "Mayotte"),
  QT_TRANSLATE_NOOP("QLocale", "Mexico"),
  QT_TRANSLATE_NOOP("QLocale", "Micronesia"),
  QT_TRANSLATE_NOOP("QLocale", "Moldova"),
  QT_TRANSLATE_NOOP("QLocale", "Monaco"),
  QT_TRANSLATE_NOOP("QLocale", "Mongolia"),
  QT_TRANSLATE_NOOP("QLocale", "Montserrat"),
  QT_TRANSLATE_NOOP("QLocale", "Morocco"),
  QT_TRANSLATE_NOOP("QLocale", "Mozambique"),
  QT_TRANSLATE_NOOP("QLocale", "Myanmar"),
  QT_TRANSLATE_NOOP("QLocale", "Namibia"),
  QT_TRANSLATE_NOOP("QLocale", "NauruCountry"),
  QT_TRANSLATE_NOOP("QLocale", "Nepal"),
  QT_TRANSLATE_NOOP("QLocale", "Netherlands"),
  QT_TRANSLATE_NOOP("QLocale", "NetherlandsAntilles"),
  QT_TRANSLATE_NOOP("QLocale", "NewCaledonia"),
  QT_TRANSLATE_NOOP("QLocale", "NewZealand"),
  QT_TRANSLATE_NOOP("QLocale", "Nicaragua"),
  QT_TRANSLATE_NOOP("QLocale", "Niger"),
  QT_TRANSLATE_NOOP("QLocale", "Nigeria"),
  QT_TRANSLATE_NOOP("QLocale", "Niue"),
  QT_TRANSLATE_NOOP("QLocale", "NorfolkIsland"),
  QT_TRANSLATE_NOOP("QLocale", "NorthernMarianaIslands"),
  QT_TRANSLATE_NOOP("QLocale", "Norway"),
  QT_TRANSLATE_NOOP("QLocale", "Oman"),
  QT_TRANSLATE_NOOP("QLocale", "Pakistan"),
  QT_TRANSLATE_NOOP("QLocale", "Palau"),
  QT_TRANSLATE_NOOP("QLocale", "PalestinianTerritory"),
  QT_TRANSLATE_NOOP("QLocale", "Panama"),
  QT_TRANSLATE_NOOP("QLocale", "PapuaNewGuinea"),
  QT_TRANSLATE_NOOP("QLocale", "Paraguay"),
  QT_TRANSLATE_NOOP("QLocale", "Peru"),
  QT_TRANSLATE_NOOP("QLocale", "Philippines"),
  QT_TRANSLATE_NOOP("QLocale", "Pitcairn"),
  QT_TRANSLATE_NOOP("QLocale", "Poland"),
  QT_TRANSLATE_NOOP("QLocale", "Portugal"),
  QT_TRANSLATE_NOOP("QLocale", "PuertoRico"),
  QT_TRANSLATE_NOOP("QLocale", "Qatar"),
  QT_TRANSLATE_NOOP("QLocale", "Reunion"),
  QT_TRANSLATE_NOOP("QLocale", "Romania"),
  QT_TRANSLATE_NOOP("QLocale", "RussianFederation"),
  QT_TRANSLATE_NOOP("QLocale", "Rwanda"),
  QT_TRANSLATE_NOOP("QLocale", "SaintKittsAndNevis"),
  QT_TRANSLATE_NOOP("QLocale", "StLucia"),
  QT_TRANSLATE_NOOP("QLocale", "StVincentAndTheGrenadines"),
  QT_TRANSLATE_NOOP("QLocale", "Samoa"),
  QT_TRANSLATE_NOOP("QLocale", "SanMarino"),
  QT_TRANSLATE_NOOP("QLocale", "SaoTomeAndPrincipe"),
  QT_TRANSLATE_NOOP("QLocale", "SaudiArabia"),
  QT_TRANSLATE_NOOP("QLocale", "Senegal"),
  QT_TRANSLATE_NOOP("QLocale", "Seychelles"),
  QT_TRANSLATE_NOOP("QLocale", "SierraLeone"),
  QT_TRANSLATE_NOOP("QLocale", "Singapore"),
  QT_TRANSLATE_NOOP("QLocale", "Slovakia"),
  QT_TRANSLATE_NOOP("QLocale", "Slovenia"),
  QT_TRANSLATE_NOOP("QLocale", "SolomonIslands"),
  QT_TRANSLATE_NOOP("QLocale", "Somalia"),
  QT_TRANSLATE_NOOP("QLocale", "SouthAfrica"),
  QT_TRANSLATE_NOOP("QLocale", "SouthGeorgiaAndTheSouthSandwichIslands"),
  QT_TRANSLATE_NOOP("QLocale", "Spain"),
  QT_TRANSLATE_NOOP("QLocale", "SriLanka"),
  QT_TRANSLATE_NOOP("QLocale", "StHelena"),
  QT_TRANSLATE_NOOP("QLocale", "StPierreAndMiquelon"),
  QT_TRANSLATE_NOOP("QLocale", "Sudan"),
  QT_TRANSLATE_NOOP("QLocale", "Suriname"),
  QT_TRANSLATE_NOOP("QLocale", "SvalbardAndJanMayenIslands"),
  QT_TRANSLATE_NOOP("QLocale", "Swaziland"),
  QT_TRANSLATE_NOOP("QLocale", "Sweden"),
  QT_TRANSLATE_NOOP("QLocale", "Switzerland"),
  QT_TRANSLATE_NOOP("QLocale", "SyrianArabRepublic"),
  QT_TRANSLATE_NOOP("QLocale", "Taiwan"),
  QT_TRANSLATE_NOOP("QLocale", "Tajikistan"),
  QT_TRANSLATE_NOOP("QLocale", "Tanzania"),
  QT_TRANSLATE_NOOP("QLocale", "Thailand"),
  QT_TRANSLATE_NOOP("QLocale", "Togo"),
  QT_TRANSLATE_NOOP("QLocale", "Tokelau"),
  QT_TRANSLATE_NOOP("QLocale", "TongaCountry"),
  QT_TRANSLATE_NOOP("QLocale", "TrinidadAndTobago"),
  QT_TRANSLATE_NOOP("QLocale", "Tunisia"),
  QT_TRANSLATE_NOOP("QLocale", "Turkey"),
  QT_TRANSLATE_NOOP("QLocale", "Turkmenistan"),
  QT_TRANSLATE_NOOP("QLocale", "TurksAndCaicosIslands"),
  QT_TRANSLATE_NOOP("QLocale", "Tuvalu"),
  QT_TRANSLATE_NOOP("QLocale", "Uganda"),
  QT_TRANSLATE_NOOP("QLocale", "Ukraine"),
  QT_TRANSLATE_NOOP("QLocale", "UnitedArabEmirates"),
  QT_TRANSLATE_NOOP("QLocale", "UnitedKingdom"),
  QT_TRANSLATE_NOOP("QLocale", "UnitedStates"),
  QT_TRANSLATE_NOOP("QLocale", "UnitedStatesMinorOutlyingIslands"),
  QT_TRANSLATE_NOOP("QLocale", "Uruguay"),
  QT_TRANSLATE_NOOP("QLocale", "Uzbekistan"),
  QT_TRANSLATE_NOOP("QLocale", "Vanuatu"),
  QT_TRANSLATE_NOOP("QLocale", "VaticanCityState"),
  QT_TRANSLATE_NOOP("QLocale", "Venezuela"),
  QT_TRANSLATE_NOOP("QLocale", "VietNam"),
  QT_TRANSLATE_NOOP("QLocale", "BritishVirginIslands"),
  QT_TRANSLATE_NOOP("QLocale", "USVirginIslands"),
  QT_TRANSLATE_NOOP("QLocale", "WallisAndFutunaIslands"),
  QT_TRANSLATE_NOOP("QLocale", "WesternSahara"),
  QT_TRANSLATE_NOOP("QLocale", "Yemen"),
  QT_TRANSLATE_NOOP("QLocale", "Yugoslavia"),
  QT_TRANSLATE_NOOP("QLocale", "Zambia"),
  QT_TRANSLATE_NOOP("QLocale", "Zimbabwe"),
};
#endif

