/****************************************************************************
**
** Copyright (C) 2009 Nokia Corporation and/or its subsidiary(-ies).
** Contact: Qt Software Information (qt-info@nokia.com)
**
** This file is part of the tools applications of the Qt Toolkit.
**
** $QT_BEGIN_LICENSE:LGPL$
** Commercial Usage
** Licensees holding valid Qt Commercial licenses may use this file in
** accordance with the Qt Commercial License Agreement provided with the
** Software or, alternatively, in accordance with the terms contained in
** a written agreement between you and Nokia.
**
** GNU Lesser General Public License Usage
** Alternatively, this file may be used under the terms of the GNU Lesser
** General Public License version 2.1 as published by the Free Software
** Foundation and appearing in the file LICENSE.LGPL included in the
** packaging of this file.  Please review the following information to
** ensure the GNU Lesser General Public License version 2.1 requirements
** will be met: http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
**
** In addition, as a special exception, Nokia gives you certain
** additional rights. These rights are described in the Nokia Qt LGPL
** Exception version 1.0, included in the file LGPL_EXCEPTION.txt in this
** package.
**
** GNU General Public License Usage
** Alternatively, this file may be used under the terms of the GNU
** General Public License version 3.0 as published by the Free Software
** Foundation and appearing in the file LICENSE.GPL included in the
** packaging of this file.  Please review the following information to
** ensure the GNU General Public License version 3.0 requirements will be
** met: http://www.gnu.org/copyleft/gpl.html.
**
** If you are unsure which license is appropriate for your use, please
** contact the sales department at qt-sales@nokia.com.
** $QT_END_LICENSE$
**
****************************************************************************/

/*
  quoter.h
*/

#ifndef QUOTER_H
#define QUOTER_H

#include <qhash.h>
#include <qstringlist.h>

#include "location.h"

QT_BEGIN_NAMESPACE

class Quoter
{
public:
    Quoter();

    void reset();
    void quoteFromFile( const QString& userFriendlyFileName,
			const QString& plainCode, const QString& markedCode );
    QString quoteLine( const Location& docLocation, const QString& command,
		       const QString& pattern );
    QString quoteTo( const Location& docLocation, const QString& command,
		     const QString& pattern );
    QString quoteUntil( const Location& docLocation, const QString& command,
			const QString& pattern );
    QString quoteSnippet(const Location &docLocation, const QString &identifier);

private:
    QString getLine();
    void failedAtEnd( const Location& docLocation, const QString& command );
    bool match( const Location& docLocation, const QString& pattern,
    		const QString& line );
    QString commentForCode() const;

    bool silent; 
    bool validRegExp;
    QStringList plainLines;
    QStringList markedLines;
    Location codeLocation;
    QHash<QString,QString> commentHash;
};

QT_END_NAMESPACE

#endif
